#!/usr/bin/env python
# Copyright 2009 David D Lowe
# released under the GPLv3
"""contains subclasses of Pigment that are meant to be subclassed themselves,
specificall FileBasedPigment , for pigment types like wallpaper and gnome splash
screens, and DirBasedPigment, for pigment types like gtk and metacity themes."""

from __future__ import division
from pigment import Pigment, PigmentType
import pigment
import pygtk
pygtk.require("2.0")
import gtk
import os, sys
from glob import glob
from gettext import gettext as _
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.insert(0,"..") # for developing purposes
import const
import handy
import ep_exceptions        

class FileBasedPigmentType(PigmentType):
    attachmentTypeHelp = "FileBasedPigmentType help"
    
    publicBaseDir = "" # eg: /usr/share/pixmaps/splash
    personalBaseDir = "" # can contain ~ which stands for the user's home directory
                         # eg: ~/.local/share/pixmaps/splash

class DirBasedPigmentType(FileBasedPigmentType):
    isDirectoryShared = False # is the directory shared with other types of pigments?
                              # eg: gtk and metacity pigments share directories
    specificChildren = ()   # if directory is shared, then each pigment should
                        # its own specific childern, eg: gtk pigments would set
                        # this to ["gtk-2.0", "index.theme"]
    

class FileBasedPigment(Pigment):
        
    def __init__(self):
        Pigment.__init__(self)
        self.type = FileBasedPigmentType()
        self.attFilename = ""
        self.attPath = ""
    
    def add_attachment(self, attachment, tempPath=None):
        if tempPath is None: 
            tempPath = os.path.join("/tmp/epidermis/pigments", self.type.directoryName,
                pigment.TEMP_DIR)
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachment) and os.path.exists(attachment):
            realPath = attachment
        else:
            if self.get_state() == pigment.STATE_INFO and len(self.installationDirectory) == 0 \
            and os.path.exists(os.path.join(tempPath,"data", os.path.basename(attachment))):
                realPath = os.path.join(tempPath, "data", os.path.basename(attachment))
            elif os.path.exists(os.path.join(self.type.publicBaseDir, os.path.basename(attachment))):
                realPath = os.path.join(self.type.publicBaseDir, os.path.basename(attachment))
            else:
                self.attPath = ""
                self.attFilename = ""
                print >> sys.stderr, "AttachmentNotFoundException, pigment state:",self.debug_get_state_str()
                print >> sys.stderr, "  pigment installationDirectory: \"%s\"" % str(self.installationDirectory)
                raise(ep_exceptions.AttachmentNotFoundException(attachment, pigment=self))
        Pigment.add_attachment(self, realPath)
        self.attPath = realPath
        self.attFilename = os.path.basename(attachment)
    
    
    def install(self, installationDirectory=None, progressCallback=None,  shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # copy attfile
        if progressCallback is None: progressCallback = lambda progress, query: False
        if os.path.exists(os.path.join(self.type.publicBaseDir,self.attFilename)):
            query = pigment.QUERY_OVERWRITE_DATA
        else:
            query = None
        if progressCallback(0.75, query): return
        shell.do(["mkdir", "-p", self.type.publicBaseDir])
        shell.do(["cp", "-f", self.attPath, os.path.join(self.type.publicBaseDir,self.attFilename)])
        shell.do(["chown", "root:root", os.path.join(self.type.publicBaseDir,self.attFilename)])
        shell.do(["chmod", "644", os.path.join(self.type.publicBaseDir, self.attFilename)])
        progressCallback(1.0, None)
            
    
    def remove(self, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, change_progress_callback, shell)
        if exitPlease: return
        
        # remove attfile
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        shell.do(["rm", "-f", os.path.join(self.type.publicBaseDir, self.attFilename)])
        progressCallback(1.0)
    
    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = os.path.dirname(filePath)
        self.add_attachment(handy.get_first_text(root.getElementsByTagName("filename")[0]), 
            tempPath = attachmentPath)
    
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        
        xmlBit = doc.createElement("filename")
        tn = doc.createTextNode(self.attFilename)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.exists(os.path.join(self.type.publicBaseDir, self.attFilename)):
                return True
        return False
    
    def is_attachment_correct(self, attachmentIndex=None):
        if attachmentIndex is None and len(self.attachments) != 1:
            return False
        if not attachmentIndex is None:
            if attachmentIndex >= len(self.attachments):
                print >> sys.stderr, "no attachment with index ", attachmentIndex
            att = self.attachments[attachmentIndex]
        else:
            att = self.attachments[0]
        if not os.path.isfile(att):
            return False
        return Pigment.is_attachment_correct(self, attachmentIndex)
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = file(self.attPath, 'rb')
            f2 = file(os.path.join(self.type.publicBaseDir, self.attFilename), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2

class DirBasedPigment(Pigment):
        
    def __init__(self):
        Pigment.__init__(self)
        self.type = DirBasedPigmentType()
        self.attDirname = ""
        self.attPath = ""
    
    def add_attachment(self, attachment, tempPath=None):
        if tempPath is None:
            tempPath = os.path.join("/tmp/epidermis/pigments/", self.type.directoryName, \
                pigment.TEMP_DIR)
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if attachment[-1] == "/": attachment = attachment[:-1]
        if os.path.isabs(attachment) and os.path.exists(attachment):
            realPath = attachment
        else:
            if self.get_state() == pigment.STATE_INFO and len(self.installationDirectory) == 0 \
            and os.path.exists(os.path.join(tempPath, "data", os.path.basename(attachment))):
                realPath = os.path.join(tempPath, "data", os.path.basename(attachment))
            elif os.path.exists(os.path.join(self.type.publicBaseDir, os.path.basename(attachment))):
                realPath = os.path.join(self.type.publicBaseDir, os.path.basename(attachment))
            else:
                self.attPath = ""
                self.attDirname = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachment, pigment=self))
        Pigment.add_attachment(self, realPath)
        self.attPath = realPath
        self.attDirname = os.path.basename(attachment)
    
    
    def install(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        if progressCallback is None: progressCallback = lambda progress, query: False
        query = None
        if os.path.exists(os.path.join(self.type.publicBaseDir, self.attDirname)):
            for tt in os.listdir(os.path.join(self.type.publicBaseDir, self.attDirname)):
                if os.path.isdir(os.path.join(self.type.publicBaseDir, self.attDirname, tt)) and tt != "cursors":
                    query = pigment.QUERY_OVERWRITE_DATA
                    break
        if progressCallback(0.5, query): return
        shell.do(["mkdir", "-p", os.path.join(self.type.publicBaseDir,self.attDirname)])
        if const.DEBUG: print "self.attPath: \"%s\"" % self.attPath
        shell.do(["cp", "-r"] + glob(self.attPath + "/*") + \
                [os.path.join(self.type.publicBaseDir,self.attDirname)])
        shell.do(["chown", "root:root", "-R", os.path.join(self.type.publicBaseDir,self.attDirname)])
        shell.do(["chmod", "755", "-R", os.path.join(self.type.publicBaseDir,self.attDirname)])
        
        alldirs = [] # save a list of all subdirectories for later
        os.path.walk(os.path.join(self.type.publicBaseDir,self.attDirname), lambda a1, dirname, a3: alldirs.append(dirname), None)
        
        shell.do(["chmod", "644", "-R", os.path.join(self.type.publicBaseDir, self.attDirname)])
        shell.do(["chmod", "755", os.path.join(self.type.publicBaseDir, self.attDirname)])
        if progressCallback(0.75, None): return
        counter = 0
        count = len(alldirs)
        for dir in alldirs:
            shell.do(["chmod", "755", dir])
            if progressCallback(0.75 + (counter / count) / 4, None): return
            counter += 1

        progressCallback(1.0, None)
            
    
    def remove(self, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # remove pigment information
        Pigment.remove(self, change_progress_callback, shell)
        if exitPlease: return
        
        # remove attDir
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        if self.type.isDirectoryShared:
            for child in self.type.specificChildren:
                shell.do(["rm", "-rf", os.path.join(self.type.publicBaseDir, self.attDirname, child)])
            if len(os.listdir(os.path.join(self.type.publicBaseDir, self.attDirname))) == 0:
                shell.do(["rm", "-rf", os.path.join(self.type.publicBaseDir, self.attDirname)])
        else:
            shell.do(["rm", "-r", os.path.join(self.type.publicBaseDir, self.attDirname)])
        progressCallback(1.0)
    
    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = os.path.dirname(filePath)
        self.add_attachment(handy.get_first_text(root.getElementsByTagName("dirname")[0]),
            tempPath = attachmentPath)
    
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        
        xmlBit = doc.createElement("dirname")
        tn = doc.createTextNode(self.attDirname)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.exists(os.path.join(self.type.publicBaseDir, self.attDirname)):
                return True
        return False
    
    def is_attachment_correct(self, attachmentIndex=None):
        if attachmentIndex is None and len(self.attachments) != 1:
            return False
        if not attachmentIndex is None:
            if attachmentIndex >= len(self.attachments):
                print >> sys.stderr, "no attachment with index ", attachmentIndex
            att = self.attachments[attachmentIndex]
        else:
            att = self.attachments[0]
        if not os.path.isdir(att):
            return False
        return Pigment.is_attachment_correct(self, attachmentIndex)
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
